
(****************************************************
   change unitname to filename
****************************************************)   
unit XXX; 

interface

uses
  classes, GrammarReader, GOLDParser, Symbol, Token;

type
  TOutputEvent = procedure (aMessage : String) of object;
  TXXXParser = class
    private
      FOnError: TOutputEvent;
      (* To use LoadGrammarFromResource, create a file with extension ".rc"
         and put the following line into it:
         
         GRAMMAR RCDATA "XXX.cgt"
         
         where XXX.cgt is your compiled grammar. Put the .cgt-File in the same
         directory the .rc-File is in. In Delphi use Menu "Project/Add to project", 
         to add the .rc-File. 
         
         Then in Method Execute uncomment the call to LoadGrammarFromResource 
         and comment the one to TGoldParser.LoadCompiledGrammar
      *)
      function LoadGrammarFromResource(aGoldParser : TGOLDParser) : Boolean;
      procedure ReplaceReduction (aParser : TGOLDParser);
    public
      procedure Execute(aSource : String);
      // Set OnError to receive ErrorMessages
      property  OnError : TOutputEvent read FOnError write FOnError;
  end;

implementation

uses
  forms,
  sysutils,
  windows;

// constant definitions - this is Delphi <= 5 compatible!
// Symbols

const
  Symbol_Eof           = 0 ; // (EOF)
  Symbol_Error         = 1 ; // (Error)
  Symbol_Whitespace    = 2 ; // (Whitespace)
  Symbol_Minus         = 3 ; // '-'
  Symbol_Percent       = 4 ; // '%'
  Symbol_Lparan        = 5 ; // '('
  Symbol_Rparan        = 6 ; // ')'
  Symbol_Times         = 7 ; // '*'
  Symbol_Div           = 8 ; // '/'
  Symbol_Plus          = 9 ; // '+'
  Symbol_Id            = 10; // Id
  Symbol_Numberliteral = 11; // NumberLiteral
  Symbol_Stringliteral = 12; // StringLiteral
  Symbol_Addexp        = 13; // <Add Exp>
  Symbol_Multexp       = 14; // <Mult Exp>
  Symbol_Negateexp     = 15; // <Negate Exp>
  Symbol_Value         = 16; // <Value>

// Rules
const
  Rule_Addexp_Plus         = 0 ; // <Add Exp> ::= <Mult Exp> '+' <Add Exp>
  Rule_Addexp_Minus        = 1 ; // <Add Exp> ::= <Mult Exp> '-' <Add Exp>
  Rule_Addexp              = 2 ; // <Add Exp> ::= <Mult Exp>
  Rule_Multexp_Times       = 3 ; // <Mult Exp> ::= <Negate Exp> '*' <Mult Exp>
  Rule_Multexp_Div         = 4 ; // <Mult Exp> ::= <Negate Exp> '/' <Mult Exp>
  Rule_Multexp_Percent     = 5 ; // <Mult Exp> ::= <Negate Exp> '%' <Mult Exp>
  Rule_Multexp             = 6 ; // <Mult Exp> ::= <Negate Exp>
  Rule_Negateexp_Minus     = 7 ; // <Negate Exp> ::= '-' <Value>
  Rule_Negateexp           = 8 ; // <Negate Exp> ::= <Value>
  Rule_Value_Id            = 9 ; // <Value> ::= Id
  Rule_Value_Stringliteral = 10; // <Value> ::= StringLiteral
  Rule_Value_Numberliteral = 11; // <Value> ::= NumberLiteral
  Rule_Value_Lparan_Rparan = 12; // <Value> ::= '(' <Add Exp> ')'


{ TXXXParser }

procedure TXXXParser.Execute(aSource : String);
var
   lParser    : TGOLDParser;
   lResponse  : Integer;
   lDone      : Boolean;
   zToken     : Integer;
   lError     : String;

begin
   lParser := TGOLDParser.Create;
   try
(************************************************************
  change filename of compiled grammar or load from resource
************************************************************)
     if lParser.LoadCompiledGrammar(ExtractFilePath(Application.ExeName)+'XXX.cgt') then begin
//     if LoadGrammarFromResource(lParser) then begin
        lParser.OpenTextString(aSource);
        lParser.TrimReductions := TRUE;
        lDone := False;
        while not lDone do begin
           lResponse := lParser.Parse;
           case lResponse of
             gpMsgLexicalError: begin
               if Assigned(FOnError) 
                 then FOnError('Line ' + IntToStr(lParser.CurrentLineNumber) + 
                               ': Lexical Error: Cannot recognize token: ' + 
                               lParser.CurrentToken.DataVar);
               lDone := True;
             end;
             gpMsgSyntaxError: begin
               if Assigned(FOnError) then begin
                 lError := '';
                 for zToken := 0 to lParser.TokenTable.Count - 1 do
                 lError := lError + ' ' + lParser.TokenTable[zToken].Name;
                 FOnError('Line ' + IntToStr(lParser.CurrentLineNumber) + 
                          ': Syntax Error: Expecting the following tokens: ' + 
                          Trim(lError));
               end; // if Assigned(FOnError)
               lDone := True;
             end;
             gpMsgReduction: begin
               ReplaceReduction(lParser);
               // Record linenumber for error messages in semantics check
               if lParser.CurrentReduction <> nil
                 then lParser.CurrentReduction.Tag := lParser.CurrentLineNumber;
             end;
             gpMsgAccept: begin
               //=== Success!
               (******************************************************************
                  enter code here
               *******************************************************************)   
               lDone := True;
             end;
             gpMsgInternalError,
             gpMsgNotLoadedError: begin
               if Assigned(FOnError) then FOnError('Internal Parser error! Aborting!');
               lDone := True;
             end;
             gpMsgCommentError: begin
               if Assigned(FOnError) 
                 then FOnError('Line ' + IntToStr(lParser.CurrentLineNumber) + 
                               ': Syntax Error: Unexpected end of file!');
               lDone := True;
             end;
           end;
        end;
     end else if Assigned(FOnError) then FOnError('Grammar could not be loaded!');
   finally
     lParser.Free;
   end; // try .. finally
end; // TXXXParser.Execute

function TXXXParser.LoadGrammarFromResource(aGoldParser: TGOLDParser) : Boolean;
var
  lMemStream : TMemoryStream;
  lResource : Pointer;
  lHandle   : Cardinal;

begin
  Result := TRUE;
  try
    lHandle := FindResource(0, 'GRAMMAR', RT_RCDATA);
    lResource := LockResource(LoadResource(0, lHandle));
    if lResource = nil then abort;
    lMemStream := TMemoryStream.Create;
    try
      lMemStream.WriteBuffer(lResource^, SizeofResource(0, lHandle));
      lMemStream.Position := 0;
      if not aGoldParser.LoadCompiledGrammar(lMemStream) then abort;
    finally
      lMemStream.Free;
    end; // try .. finally
  except
    Result := FALSE;
  end; // try .. except
end; // TXXXParser.LoadGrammarFromResource

procedure TXXXParser.ReplaceReduction(aParser: TGOLDParser);
begin
  case aParser.CurrentReduction.ParentRule.TableIndex of
    Rule_Addexp_Plus         : begin
      
    end; // <Add Exp> ::= <Mult Exp> '+' <Add Exp>                  
    Rule_Addexp_Minus        : begin
      
    end; // <Add Exp> ::= <Mult Exp> '-' <Add Exp>                  
    Rule_Addexp              : begin
      
    end; // <Add Exp> ::= <Mult Exp>                  
    Rule_Multexp_Times       : begin
      
    end; // <Mult Exp> ::= <Negate Exp> '*' <Mult Exp>                  
    Rule_Multexp_Div         : begin
      
    end; // <Mult Exp> ::= <Negate Exp> '/' <Mult Exp>                  
    Rule_Multexp_Percent     : begin
      
    end; // <Mult Exp> ::= <Negate Exp> '%' <Mult Exp>                  
    Rule_Multexp             : begin
      
    end; // <Mult Exp> ::= <Negate Exp>                  
    Rule_Negateexp_Minus     : begin
      
    end; // <Negate Exp> ::= '-' <Value>                  
    Rule_Negateexp           : begin
      
    end; // <Negate Exp> ::= <Value>                  
    Rule_Value_Id            : begin
      
    end; // <Value> ::= Id                  
    Rule_Value_Stringliteral : begin
      
    end; // <Value> ::= StringLiteral                  
    Rule_Value_Numberliteral : begin
      
    end; // <Value> ::= NumberLiteral                  
    Rule_Value_Lparan_Rparan : begin
      
    end; // <Value> ::= '(' <Add Exp> ')'                  
  end; // case aParser.CurrentReduction.ParentRule.TableIndex
end; // TXXXParser.ReplaceReduction

end.
